@@ -18,7 +18,7 @@
 #include "eog-hig-dialog.h"
 #include "eog-pixbuf-util.h"
 
-static char* last_dir[] = { NULL, NULL };
+static char* last_dir[] = { NULL, NULL, NULL, NULL };
 
 #define FILE_FORMAT_KEY "file-format"
 
@@ -152,37 +152,41 @@ response_cb (GtkDialog *dlg, gint id, gpointer data)
 }
 
 static void
-eog_file_selection_add_filter (GtkWidget *widget)
+eog_file_selection_add_filter (EogFileSelection *filesel)
 {
-	EogFileSelection *filesel;
 	GSList *it;
 	GSList *formats;
-	GtkFileFilter *filter;
-	GtkFileFilter *all_img_filter;
  	GtkFileFilter *all_file_filter;
+	GtkFileFilter *all_img_filter;
+	GtkFileFilter *filter;
+	GSList *filters = NULL;
 	gchar **mime_types, **pattern, *tmp;
 	int i;
+	GtkFileChooserAction action;
 
-	filesel = EOG_FILE_SELECTION (widget);
+	action = gtk_file_chooser_get_action (GTK_FILE_CHOOSER (filesel));
+
+	if (action != GTK_FILE_CHOOSER_ACTION_SAVE && action != GTK_FILE_CHOOSER_ACTION_OPEN) {
+		return;
+	}
 
 	/* All Files Filter */
 	all_file_filter = gtk_file_filter_new ();
 	gtk_file_filter_set_name (all_file_filter, _("All Files"));
 	gtk_file_filter_add_pattern (all_file_filter, "*");
-	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (filesel), all_file_filter);
 
 	/* All Image Filter */
 	all_img_filter = gtk_file_filter_new ();
 	gtk_file_filter_set_name (all_img_filter, _("All Images"));
-	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (filesel), all_img_filter);
 
-	if (gtk_file_chooser_get_action (GTK_FILE_CHOOSER (filesel)) == GTK_FILE_CHOOSER_ACTION_SAVE) {
+	if (action == GTK_FILE_CHOOSER_ACTION_SAVE) {
 		formats = eog_pixbuf_get_savable_formats ();
 	}
 	else {
 		formats = gdk_pixbuf_get_formats ();
 	}
 
+	/* Image filters */
 	for (it = formats; it != NULL; it = it->next) {
 		char *filter_name;
 		GdkPixbufFormat *format;
@@ -196,7 +200,7 @@ eog_file_selection_add_filter (GtkWidget *widget)
 					       gdk_pixbuf_format_get_name (format));
 		gtk_file_filter_set_name (filter, filter_name);
 		g_free (filter_name);
-		
+
 		mime_types = gdk_pixbuf_format_get_mime_types ((GdkPixbufFormat *) it->data);
 		for (i = 0; mime_types[i] != NULL; i++) {
 			gtk_file_filter_add_mime_type (filter, mime_types[i]);
@@ -218,13 +222,20 @@ eog_file_selection_add_filter (GtkWidget *widget)
 		g_object_set_data (G_OBJECT (filter), 
 				   FILE_FORMAT_KEY,
 				   format);
-
-		gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (filesel), filter);
 		
+		filters = g_slist_prepend (filters, filter);
 	}
 	g_slist_free (formats);
 
+	/* Add filter to filechooser */
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (filesel), all_file_filter);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (filesel), all_img_filter);
 	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (filesel), all_img_filter);
+
+	for (it = filters; it != NULL; it = it->next) {
+		gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (filesel), GTK_FILE_FILTER (it->data));
+	}
+	g_slist_free (filters);
 }
 
 static void
@@ -425,8 +436,7 @@ eog_file_selection_new (GtkFileChooserAction action)
 
 	filesel = g_object_new (EOG_TYPE_FILE_SELECTION,
 				"action", action,
-				"select-multiple", FALSE /* (action == GTK_FILE_CHOOSER_ACTION_OPEN) */,
-				"folder-mode", FALSE,
+				"select-multiple", (action == GTK_FILE_CHOOSER_ACTION_OPEN),
 				NULL);
 	switch (action) {
 	case GTK_FILE_CHOOSER_ACTION_OPEN:
@@ -445,12 +455,22 @@ eog_file_selection_new (GtkFileChooserAction action)
 		title = _("Save Image");
 		break;
 
+	case GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER:
+		gtk_dialog_add_buttons (GTK_DIALOG (filesel),
+					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					GTK_STOCK_OPEN, GTK_RESPONSE_OK,
+					NULL);
+		title = _("Open Folder");
+		break;
+
 	default:
 		g_assert_not_reached ();
 	}
 
-	eog_file_selection_add_filter (filesel);
-	eog_file_selection_add_preview (filesel);
+	if (action != GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER) {
+		eog_file_selection_add_filter (EOG_FILE_SELECTION (filesel));
+		eog_file_selection_add_preview (filesel);
+	}
 
 	if (last_dir[action] != NULL) {
 		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (filesel), last_dir [action]);
@@ -463,33 +483,6 @@ eog_file_selection_new (GtkFileChooserAction action)
 	return filesel;
 }
 
-GtkWidget* 
-eog_folder_selection_new (void)
-{
-	GtkWidget *filesel;
-
-	filesel = g_object_new (EOG_TYPE_FILE_SELECTION,
-				"action", GTK_FILE_CHOOSER_ACTION_OPEN,
-				"select-multiple", FALSE,
-				"folder-mode", TRUE,
-				NULL);
-	gtk_dialog_add_buttons (GTK_DIALOG (filesel),
-				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-				GTK_STOCK_OPEN, GTK_RESPONSE_OK,
-				NULL);
-
-	if (last_dir[GTK_FILE_CHOOSER_ACTION_OPEN] != NULL) {
-		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (filesel), 
-						     last_dir [GTK_FILE_CHOOSER_ACTION_OPEN]);
-	}
-
-	g_signal_connect (G_OBJECT (filesel), "response", G_CALLBACK (response_cb), NULL);
-
-	gtk_window_set_title (GTK_WINDOW (filesel), _("Open Folder"));
-	gtk_dialog_set_default_response (GTK_DIALOG (filesel), GTK_RESPONSE_OK);
-
-	return filesel;
-}
 
 GdkPixbufFormat* 
 eog_file_selection_get_format (EogFileSelection *sel)
